#ifndef	_PLK_SCANSDK_H_
#define _PLK_SCANSDK_H_

#ifdef __cplusplus
extern "C" {
#endif


#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sys/time.h>
#include <dirent.h>
#include <stdint.h>
#include <dlfcn.h>
#include "libusb-1.0/libusb.h"
#include "sane/sane.h"
#include "plk_secure_ctl.h"

#define PLK_SCANSDK_API extern __attribute__((visibility("default")))

#define DWORD	uint32_t /*unsigned long */
#define WORD	uint16_t /*unsigned short*/
#define LONG	int32_t /*long*/
#define BYTE	unsigned char
#define SHORT	int16_t /*short*/
#define BOOL	int32_t /*int*/
#define PVOID	void *
#define PBYTE   unsigned char *
#define LPVOID	PVOID
#define LPBYTE	PBYTE
#define PDWORD	DWORD *
#define UINT	uint32_t /*unsigned int*/
#define	PUCHAR	BYTE *
#define TCHAR	char
//#define HANDLE	WORD //delete by matt0326 b'x no use in PSS and conflict with grg's HANDLE
#define PWORD	WORD *
#define MAX_PATH	1024
#define FALSE       0
#define TRUE        1
#define LPTSTR      char *
#define LPCTSTR		char *
#define LPDWORD		PDWORD
#define LPWORD      PWORD
#define	LPINT		int32_t *
#define INVALID_HANDLE_VALUE	-1
#define HINSTANCE	int32_t
#define MAXLENGTH 20480+1
#define MAX_FILEPATH	2048
#define MAX_LIST_LENGTH	512000
#define LIBUSBX_API_MIN_VERSION 0x01000102

//VTM API 2.0
#define MAX_STRING_NUMBER		60
#define BARCODE_MAX_LENGTH		2048
#define BARCODE_BIG_MAX_LENGTH	8192

typedef enum _ENUM_PLKSS_ERRCODE
{
	/*Lisence STATUS*/
	PLKSS_ERRCODE_LIC_REGISTER_FAIL = -500,
	PLKSS_ERRCODE_LIC_UNREGISTER_FAIL,
	PLKSS_ERRCODE_LIC_STATUS_FAILED,
	PLKSS_ERRCODE_LIC_PERMISSION_FAILED,

	/*VTM STATUS*/
	PLKSS_ERRCODE_VTM_SCAN_OVERTIME = -400,
	PLKSS_ERRCODE_VTM_GRATING_SENSOR_NOT_MOVE,
	PLKSS_ERRCODE_VTM_NO_DEVICES_AFTER_EJECT,
	PLKSS_ERRCODE_VTM_BOTH_SIDE_HAVE_PAPER,
	/*CallBack STATUS*/
	PLKSS_ERRCODE_CB_STATUS_IL_READ_IMAGE_FAIL = -300,
	PLKSS_ERRCODE_CB_STATUS_IL_AUTO_DESKEW_EX_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_AUTO_CROP_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_PUNCH_HOLE_REMOVAL_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_REMOVE_BLANK_PAGE_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_APPLY_CONTRAST_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_APPLY_GAMMA_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_APPLY_BRIGHTNESS_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_REDUCE_BORDER_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_AUTO_DENSITY_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_AUTO_ENHANCE_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_REMOVE_BACKGROUND_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_CHARACTER_ENHANCEMENT_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_CONVERT_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_SAVE_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_RESIZE_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_ROTATE_FAIL,
	PLKSS_ERRCODE_CB_STATUS_MERGE_FAIL,
	PLKSS_ERRCODE_CB_STATUS_MERGE_OUT_OF_MEMORY,
	PLKSS_ERRCODE_CB_STATUS_MERGE_FILE_NOT_EXIST,
	PLKSS_ERRCODE_CB_STATUS_IL_APPLY_HISTOGRAM_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_RESIZE_BY_RESOLUTION_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_APPLY_SHARPEN_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_AUTO_ROTATE_INTERFACE_FAIL,
	PLKSS_ERRCODE_CB_STATUS_IL_RED_TYPE_ENHANCEMENT_FAIL, //Joseph Add 2022/03/29
	/*PAPER STATUS*/
	PLKSS_ERRCODE_PAPER_STATUS_NO_PAPER = -200,
	PLKSS_ERRCODE_PAPER_STATUS_ON_DOCUMENT_TRAY,
	PLKSS_ERRCODE_PAPER_STATUS_LOADING,
	PLKSS_ERRCODE_PAPER_STATUS_EJECTING,
	PLKSS_ERRCODE_PAPER_STATUS_JAM,
	PLKSS_ERRCODE_PAPER_STATUS_ERROR_FEEDING,
	PLKSS_ERRCODE_PAPER_STATUS_COVER_OPENED,
	PLKSS_ERRCODE_PAPER_STATUS_SCANNING,
	PLKSS_ERRCODE_PAPER_STATUS_MULTIFEED,
	PLKSS_ERRCODE_PAPER_STATUS_XFER_CONDITION,
	/*SDK Function Return Code*/
	PLKSS_ERRCODE_FAIL = -100,                 /**< The function is failed */
	PLKSS_ERRCODE_NO_INIT,                     /**< Not do INIT */
	PLKSS_ERRCODE_NOT_YET_OPEN_DEVICE,         /**< Not do OPEN_DEVICE */
	PLKSS_ERRCODE_DEVICE_ALREADY_OPEN,         /**< The device has opened by OPEN_DEVICE */
	PLKSS_ERRCODE_INVALID_SOURCE,              /**< Input invalid source */
	PLKSS_ERRCODE_ONLY_SUPPORT_COLOR_MODE,     /**< Only support color mode */
	PLKSS_ERRCODE_NO_SUPPORT_EJECT,            /**< No support eject direction control */
	PLKSS_ERRCODE_PAPER_NOT_READY,             /**< No paper */
	PLKSS_ERRCODE_INVALID_SERIALNUM,	   /**< The Serial number is invailid */
	PLKSS_ERRCODE_FORMAT_NOT_SUPPORT,          /**< The format is not supported */
	PLKSS_ERRCODE_NO_CALIBRATION_DATA,         /**< Not yet calibration */
	PLKSS_ERRCODE_NO_DEVICES,
	PLKSS_ERRCODE_NO_DEVICE_NAME,
	PLKSS_ERRCODE_NO_SOURCE,
	PLKSS_ERRCODE_FILE_NO_EXIST,
	PLKSS_ERRCODE_FUNCTION_NOT_SUPPORT,
	PLKSS_ERRCODE_INVALID_PARAM,
	PLKSS_ERRCODE_NO_SCAN_PARAM,				/**< Not do  */
	PLKSS_ERRCODE_OPEN_FAIL_INVALID_HANDLE,		/* open fail with invalid handle */
	PLKSS_ERRCODE_SUCCESS = 0,                 /**< The function is performed successfully */
	PLKSS_ERRCODE_SANE_STATUS_UNSUPPORTED,	/* operation is not supported */
	PLKSS_ERRCODE_SANE_STATUS_CANCELLED,	/* operation was cancelled */
	PLKSS_ERRCODE_SANE_STATUS_DEVICE_BUSY,	/* device is busy; try again later */
	PLKSS_ERRCODE_SANE_STATUS_INVAL,		/* data is invalid (includes no dev at open) */
	PLKSS_ERRCODE_SANE_STATUS_EOF,		/* no more data available (end-of-file) */
	PLKSS_ERRCODE_SANE_STATUS_JAMMED,		/* document feeder jammed */
	PLKSS_ERRCODE_SANE_STATUS_NO_DOCS,	/* document feeder out of documents */
	PLKSS_ERRCODE_SANE_STATUS_COVER_OPEN,	/* scanner cover is open */
	PLKSS_ERRCODE_SANE_STATUS_IO_ERROR,	/* error during device I/O */
	PLKSS_ERRCODE_SANE_STATUS_NO_MEM,		/* out of memory */
	PLKSS_ERRCODE_SANE_STATUS_ACCESS_DENIED,	/* access to resource has been denied */
	PLKSS_ERRCODE_PAPER_STATUS_READY = 200,
	PLKSS_ERRCODE_CB_STATUS_SUCCESS = 300,
	PLKSS_ERRCODE_CB_STATUS_IL_REMOVE_BLANK_PAGE_SUCCESS,
	PLKSS_ERRCODE_CB_STATUS_MERGE_SUCCESS,
	PLKSS_ERRCODE_CB_STATUS_SUCCESS_BUT_ITS_BLANK_PAGE,
	PLKSS_ERRCODE_VTM_PS_DEV_READY_NO_PAPER = 400,
	PLKSS_ERRCODE_VTM_PS_READY_TO_SCAN,
	PLKSS_ERRCODE_VTM_PS_READY_TO_EJECT,
	PLKSS_ERRCODE_VTM_PS_FRONT_AND_BACK_SENSOR_HAVE_PAPER_READY
} ENUM_PLKSS_ERRCODE;

typedef	enum tagPAPER_STATUS
{
	PAPER_STATUS_NO_PAPER = 0,
	PAPER_STATUS_READY,
	PAPER_STATUS_ON_DOCUMENT_TRAY,
	PAPER_STATUS_LOADING,
	PAPER_STATUS_EJECTING,
	PAPER_STATUS_JAM,
	PAPER_STATUS_ERROR_FEEDING,
	PAPER_STATUS_COVER_OPENED,
	PAPER_STATUS_SCANNING,
	PAPER_STATUS_MULTIFEED,
	PAPER_STATUS_XFER_CONDITION,
	PAPER_STATUS_LENGTH_MULTIFEED,
	PAPER_STATUS_PAPER_OUT_SHORT_PAPER,	
	PAPER_STATUS_PAPER_OUT_LONG_PAPER,	
	PAPER_STATUS_PAPER_SKEW,
	PAPER_STATUS_PAPER_NOSKEW
} PLKSS_PAPER_STATUS;

typedef enum tagEJECT_DIRECTION
{
	EJECT_FORWARD = 1,
	EJECT_BACKWARD,
	EJECT_BWD_STOP,
	EJECT_FWD_STOP
} EJECT_DIRECTION;

typedef enum tagFILE_TYPE
{
	FT_JPG,
	FT_BMP,
	FT_TIF,
	FT_PNG,
	FT_PDF,
	FT_OFD,
	FT_MPDF,
	FT_MOFD,
	FT_MTIF,
	FT_PNM,
	FT_OCRTXT,
	FT_OCRDOC,
	FT_SPDF,
	FT_SOFD,
	FT_FORMAT_END
} FILE_TYPE;

typedef	enum tagCB_STATUS
{
	CB_STATUS_IL_READ_IMAGE_FAIL = -300,
	CB_STATUS_IL_AUTO_DESKEW_EX_FAIL,
	CB_STATUS_IL_AUTO_CROP_FAIL,
	CB_STATUS_IL_PUNCH_HOLE_REMOVAL_FAIL,
	CB_STATUS_IL_REMOVE_BLANK_PAGE_FAIL,
	CB_STATUS_IL_APPLY_CONTRAST_FAIL,
	CB_STATUS_IL_APPLY_GAMMA_FAIL,
	CB_STATUS_IL_APPLY_BRIGHTNESS_FAIL,
	CB_STATUS_IL_REDUCE_BORDER_FAIL,
	CB_STATUS_IL_AUTO_DENSITY_FAIL,
	CB_STATUS_IL_AUTO_ENHANCE_FAIL,
	CB_STATUS_IL_REMOVE_BACKGROUND_FAIL,
	CB_STATUS_IL_CHARACTER_ENHANCEMENT_FAIL,
	CB_STATUS_IL_CONVERT_FAIL,
	CB_STATUS_IL_SAVE_FAIL,
	CB_STATUS_IL_RESIZE_FAIL,
	CB_STATUS_IL_ROTATE_FAIL,
	CB_STATUS_MERGE_FAIL,
	CB_STATUS_MERGE_OUT_OF_MEMORY,
	CB_STATUS_MERGE_FILE_NOT_EXIST,
	CB_STATUS_IL_APPLY_HISTOGRAM_FAIL,
	CB_STATUS_IL_RESIZE_BY_RESOLUTION_FAIL,
	CB_STATUS_IL_APPLY_SHARPEN_FAIL,
	CB_STATUS_IL_AUTO_ROTATE_INTERFACE_FAIL,
	CB_STATUS_IL_RED_TYPE_ENHANCEMENT_FAIL, //Joseph Add 2022/03/29
	CB_STATUS_SUCCESS = 300,
	CB_STATUS_IL_REMOVE_BLANK_PAGE_SUCCESS,
	CB_STATUS_MERGE_SUCCESS,
	CB_STATUS_SUCCESS_BUT_ITS_BLANK_PAGE,
} CB_STATUS;

typedef enum tagVTM_SPEED
{
	VTM_PS_FASTEST=0,
	VTM_PS_FAST,
	VTM_PS_QUICK,
	VTM_PS_STANDARD,
	VTM_PS_SLOW,
	VTM_PS_SLOWEST,
	VTM_PS_MAXVALUE
} VTM_SPEED;

typedef enum tagSCANNER_TYPE
{
	ST_DOCUMENT=0,
	ST_SECURE,
	ST_MAXVALUE
} SCANNER_TYPE;

typedef enum _ENUM_MULTIFEED_ACTION
{
	ACTION_STOP = 0,
	ACTION_CONTINUE,
	ACTION_CONFIRM
} ENUM_MULTIFEED_ACTION;

typedef enum _ENUM_MULTIFEED_REACTION
{
	REACTION_CANCEL = 0,
	REACTION_CONTINUE,
	REACTION_RETRY
} ENUM_MULTIFEED_REACTION;

typedef struct
{
	char model_name[8];	/* unique device name */
}SCANNER_NAME;

typedef struct tagSCANNERINFO
{
	char		device_name[128];
	char		scanner_name [128];
}SCANNERINFO;

typedef struct tagDeviceSerialNumber /* 20220914 claire+ */
{
	char		device_name[256];
	char		device_serial_number[256];
} DEVICE_SERIAL_NUMBER;

typedef struct
{
	int iCounter;
	CB_STATUS iStatus;
	int iEffectivePages;
	char szFileName[MAX_PATH];
}CB_INFO;

typedef struct tagDeviceIDList{
	char	str_model_name[15];
	char	str_dev_name[10];
	int 	w_product;
} DEVICE_ID_LIST;

typedef struct tagDeviceMapping{
	char	str_device_no[10];
	char	str_serial_number[32]; //20221025 16 -> 32
	BOOL	b_conform;
} DEVICE_MAPPING;

typedef struct tagVTM_BARCODE_INFO_O
{
	int inumber;
	TCHAR Type[1024];
	TCHAR szText[2048];
} VTM_BARCODE_INFO_O, *PVTM_BARCODE_INFO_O;

typedef struct tagVTM_SPEEDPARAM{
	VTM_SPEED iFeedSpeed;
	VTM_SPEED iFejectSpeed;
	VTM_SPEED iRejectSpeed;
} VTM_SPEEDPARAM, *PVTM_SPEEDPARAM;

typedef struct tagVTM_BARCODEDATA
{
	WORD iicount1;
	PVTM_BARCODE_INFO_O BarcodeInfo1;
	WORD iicount2;
	PVTM_BARCODE_INFO_O BarcodeInfo2;
} VTM_BARCODEDATA, *PVTM_BARCODEDATA;

typedef struct tagSCANNERINFOEX
{
	char		device_name[128];
	char		scanner_name [128];
	char		serial_number[16];
	int 		idVendor;
	int 		idProduct;
	int 		scanner_type;
}SCANNERINFOEX;

typedef enum tagPSS_BARCODE_TYPE
{
	API_BARCODE_TYPE_1D = 0,
	API_BARCODE_TYPE_2D,
	API_BARCODE_TYPE_1D_2D,
	API_BARCODE_TYPE_QRCODE,
	API_BARCODE_TYPE_PDF417,
	API_BARCODE_TYPE_ALL
} PSS_BARCODE_TYPE;

//VTM API 2.0
typedef enum tagVTM_OPEN_LEVEL
{
	VTM_OPEN_LEVEL_1 = 0,
	VTM_OPEN_LEVEL_2
} VTM_OPEN_LEVEL;

typedef enum tagVTM_SCAN_DIRECTION
{
	VTM_SCAN_DIRECTION_FORWARD = 0,
	VTM_SCAN_DIRECTION_BACKWARD
} VTM_SCAN_DIRECTION;

typedef enum tagVTM_EJECT_TYPE
{
	VTM_EJECT_TYPE_FORWARD = 0,
	VTM_EJECT_TYPE_BACKWARD,
	VTM_EJECT_TYPE_FORWARD_STOP
} VTM_EJECT_TYPE;

#define VTM_EJECT_TYPE_BACKWARD_STOP 3

typedef enum tagVTM_SENSOR_TYPE
{
	VTM_SENSOR_TYPE_ALL = 0,
	VTM_SENSOR_TYPE_COVER,
	VTM_SENSOR_TYPE_FRONT,
	VTM_SENSOR_TYPE_BACK,
	VTM_SENSOR_TYPE_LEFT,
	VTM_SENSOR_TYPE_RIGHT,
	VTM_SENSOR_TYPE_SCAN
} VTM_SENSOR_TYPE;

typedef enum tagVTM_SCAN_TYPE
{
	VTM_SCAN_TYPE_FRONT = 0,
	VTM_SCAN_TYPE_REAR,
	VTM_SCAN_TYPE_DUPLEX
} VTM_SCAN_TYPE;

typedef enum tagVTM_SCAN_MODE
{
	VTM_SCAN_MODE_BW = 0,
	VTM_SCAN_MODE_GRAY,
	VTM_SCAN_MODE_COLOR
} VTM_SCAN_MODE;

typedef enum tagVTM_SCAN_RESOLUTION
{
	VTM_SCAN_RESOLUTION_100 = 0,
	VTM_SCAN_RESOLUTION_150,
	VTM_SCAN_RESOLUTION_200,
	VTM_SCAN_RESOLUTION_300,
	VTM_SCAN_RESOLUTION_600
} VTM_SCAN_RESOLUTION;

typedef enum tagVTM_SCAN_FORMAT
{
	VTM_SCAN_FORMAT_JPG = 0,
	VTM_SCAN_FORMAT_BMP,
	VTM_SCAN_FORMAT_TIF,
	VTM_SCAN_FORMAT_PDF
} VTM_SCAN_FORMAT;

typedef enum tagVTM_SPEED_TYPE
{
	VTM_SPEED_TYPE_EJECT_FORWARD = 0,
	VTM_SPEED_TYPE_EJECT_BACKWARD,
	VTM_SPEED_TYPE_FEEDING,
	VTM_SPEED_TYPE_COUNT
} VTM_SPEED_TYPE;

typedef enum tagVTM_SPEED_SPEED
{
	VTM_SPEED_SPEED_FASTEST = 0,
	VTM_SPEED_SPEED_FAST,
	VTM_SPEED_SPEED_QUICK,
	VTM_SPEED_SPEED_STANDARD,
	VTM_SPEED_SPEED_SLOW,
	VTM_SPEED_SPEED_SLOWEST
} VTM_SPEED_SPEED;

typedef	enum tagVTM_RET
{
	VTM_RET_SUCCESS	= 0,
	VTM_RET_FAILED, //1
	VTM_RET_DRIVER_NOT_FOUND, //2
	VTM_RET_DRIVER_LOAD_FAILED, //3
	VTM_RET_LIBRARY_LOAD_FAILED, //4
	VTM_RET_DEVICE_NOT_FOUND, //5
	VTM_RET_NO_CALIBRATION_DATA, //6
	VTM_RET_NO_OPEN_SCANNER, //7
	VTM_RET_BAD_FUNCTION, //8
	VTM_RET_COVER_OPENED, //9
	VTM_RET_PAPER_FEEDING_ERROR, //10
	VTM_RET_PAPER_JAM, //11
	VTM_RET_SCAN_TIMEOUT, //12
	VTM_RET_REOPEN_ERROR, //13
	VTM_RET_PAPER_MULTIFEED	//14
} VTM_RET;

typedef	enum tagVTM_STATUS_RET
{
	VTM_STATUS_RET_UNKNOWN = 0,
	VTM_STATUS_RET_NONE,
	VTM_STATUS_RET_NONE_FRONT,
	VTM_STATUS_RET_NONE_BACK,
	VTM_STATUS_RET_NONE_FRONT_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_NONE_BACK_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_NONE_FRONT_BACK_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_NONE_FRONT_SCAN,
	VTM_STATUS_RET_NONE_FRONT_BACK_SCAN,
	VTM_STATUS_RET_NONE_BACK_SCAN,
	VTM_STATUS_RET_YELLOW,
	VTM_STATUS_RET_YELLOW_COVER,
	VTM_STATUS_RET_YELLOW_COVER_FRONT,
	VTM_STATUS_RET_YELLOW_COVER_BACK,
	VTM_STATUS_RET_YELLOW_COVER_FRONT_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_YELLOW_COVER_BACK_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_YELLOW_COVER_FRONT_BACK_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_YELLOW_COVER_FRONT_SCAN,
	VTM_STATUS_RET_YELLOW_COVER_FRONT_BACK_SCAN,
	VTM_STATUS_RET_YELLOW_COVER_BACK_SCAN,
	VTM_STATUS_RET_BLUE,
	VTM_STATUS_RET_BLUE_FRONT,
	VTM_STATUS_RET_BLUE_BACK,
	VTM_STATUS_RET_BLUE_FRONT_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_BLUE_BACK_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_BLUE_FRONT_BACK_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_BLUE_FRONT_SCAN,
	VTM_STATUS_RET_BLUE_FRONT_BACK_SCAN,
	VTM_STATUS_RET_BLUE_BACK_SCAN,
	VTM_STATUS_RET_NONE_FRONT_LEFT_SCAN,
	VTM_STATUS_RET_NONE_FRONT_RIGHT_SCAN,
	VTM_STATUS_RET_YELLOW_COVER_FRONT_LEFT_SCAN,
	VTM_STATUS_RET_YELLOW_COVER_FRONT_RIGHT_SCAN,
	VTM_STATUS_RET_BLUE_FRONT_LEFT_SCAN,
	VTM_STATUS_RET_BLUE_FRONT_RIGHT_SCAN,
	VTM_STATUS_RET_YELLOW_BLINKING,
	VTM_STATUS_RET_YELLOW_BLINKING_FRONT,
	VTM_STATUS_RET_YELLOW_BLINKING_BACK,
	VTM_STATUS_RET_YELLOW_BLINKING_FRONT_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_YELLOW_BLINKING_BACK_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_YELLOW_BLINKING_FRONT_BACK_LEFT_RIGHT_SCAN,
	VTM_STATUS_RET_YELLOW_BLINKING_FRONT_SCAN,
	VTM_STATUS_RET_YELLOW_BLINKING_FRONT_BACK_SCAN,
	VTM_STATUS_RET_YELLOW_BLINKING_BACK_SCAN,
	VTM_STATUS_RET_YELLOW_BLINKING_FRONT_LEFT_SCAN,
	VTM_STATUS_RET_YELLOW_BLINKING_FRONT_RIGHT_SCAN
} VTM_STATUS_RET;

typedef enum tagVTM_BARCODE_TYPE
{
	VTM_BARCODE_TYPE_3OF9 = 0,
	VTM_BARCODE_TYPE_2OF5INTERLEAVED,
	VTM_BARCODE_TYPE_2OF5NONINTERLEAVED,
	VTM_BARCODE_TYPE_CODE93,
	VTM_BARCODE_TYPE_CODE128,
	VTM_BARCODE_TYPE_UCC128,
	VTM_BARCODE_TYPE_CODABAR,
	VTM_BARCODE_TYPE_UPCA,
	VTM_BARCODE_TYPE_UPCE,
	VTM_BARCODE_TYPE_EAN8,
	VTM_BARCODE_TYPE_EAN13,
	VTM_BARCODE_TYPE_POSTNET,
	VTM_BARCODE_TYPE_RESERVED,
	VTM_BARCODE_TYPE_2OF5INDUSTRIAL,
	VTM_BARCODE_TYPE_2OF5MATRIX,
	VTM_BARCODE_TYPE_2OF5DATALOGIC,
	VTM_BARCODE_TYPE_2OF5IATA,
	VTM_BARCODE_TYPE_3OF9FULLASCII,
	VTM_BARCODE_TYPE_CODABARWITHSTARTSTOP,
	VTM_BARCODE_TYPE_MAXICODE,
	VTM_BARCODE_TYPE_QRCODE,
	VTM_BARCODE_TYPE_DATAMATRIX,
	VTM_BARCODE_TYPE_AZTEC,
	VTM_BARCODE_TYPE_UNSUPPORTED
} VTM_BARCODE_TYPE;

typedef enum tagVTM_BARCODE_CATEGORY
{
	VTM_BARCODE_CATEGORY_ALL = 0,
	VTM_BARCODE_CATEGORY_1D,
	VTM_BARCODE_CATEGORY_2D,
} VTM_BARCODE_CATEGORY;

typedef enum tagVTM_DEVICE_PARAM_TYPE
{
	VTM_DEVICE_PARAM_SPEED = 0,
	VTM_DEVICE_PARAM_EJECT,
	VTM_DEVICE_PARAM_MOTOR
} VTM_DEVICE_PARAM_TYPE;

typedef enum tagVTM_STATUS_LED
{
	VTM_STATUS_LED_NONE = 0,
	VTM_STATUS_LED_YELLOW,
	VTM_STATUS_LED_YELLOW_BLINKING,
	VTM_STATUS_LED_BLUE,
	VTM_STATUS_LED_BLUE_BLINKING
} VTM_STATUS_LED;

typedef enum tagVTM_STATUS_SKEW
{
	VTM_STATUS_SKEW_SKEW = 0,
	VTM_STATUS_SKEW_NO_SKEW
} VTM_STATUS_SKEW;

typedef enum tagVTM_STATUS_SENSOR_NAME
{
	VTM_STATUS_SENSOR_COVER = 0,
	VTM_STATUS_SENSOR_FRONT,
	VTM_STATUS_SENSOR_BACK,
	VTM_STATUS_SENSOR_LEFT,
	VTM_STATUS_SENSOR_RIGHT,
	VTM_STATUS_SENSOR_SCAN,
	VTM_STATUS_SENSOR_COUNT
} VTM_STATUS_SENSOR_NAME;

typedef enum tagVTM_STATUS_SENSOR_TYPE
{
	VTM_STATUS_SENSOR_TRIGGERED = 0,
	VTM_STATUS_SENSOR_NOT_TRIGGERED
} VTM_STATUS_SENSOR_TYPE;

typedef enum tagVTM_STATUS_HARDWARE_NAME
{
	VTM_STATUS_HARDWARE_SYSTEM = 0,
	VTM_STATUS_HARDWARE_SDRAM,
	VTM_STATUS_HARDWARE_GAMMARAM,
	VTM_STATUS_HARDWARE_SERIALFLASH,
	VTM_STATUS_HARDWARE_USBDEVICE,
	VTM_STATUS_HARDWARE_PROGRAMROM,
	VTM_STATUS_HARDWARE_COUNT
} VTM_STATUS_HARDWARE_NAME;

typedef enum tagVTM_STATUS_HARDWARE_TYPE
{
	VTM_STATUS_HARDWARE_NORMAL = 0,
	VTM_STATUS_HARDWARE_EXCEPTION
} VTM_STATUS_HARDWARE_TYPE;

typedef enum tagVTM_MOTOR_ACTION
{
	VTM_MOTOR_ACTION_STOP = 0,
	VTM_MOTOR_ACTION_START
} VTM_MOTOR_ACTION;

typedef struct tagVTM_VERSION_INFO
{
	TCHAR	szFirmwareVersion[MAX_STRING_NUMBER];
	TCHAR	szScanAPIVersion[MAX_STRING_NUMBER];
	TCHAR   szPLKVTMVersion[MAX_STRING_NUMBER];
	TCHAR	szSerialNumber[MAX_STRING_NUMBER];
} VTM_VERSION_INFO, *PVTM_VERSION_INFO;

typedef struct tagVTM_STATUS_SPEED
{
	VTM_SPEED_SPEED			wSpeed[VTM_SPEED_TYPE_COUNT];
} VTM_STATUS_SPEED, *PVTM_STATUS_SPEED;

typedef	struct tagVTM_SCANAPI_DEVICE_SENSORSTATUS
{
	DWORD		Sensor1;
	DWORD		Sensor2;
	DWORD		Sensor3;
	DWORD		Sensor4;
	DWORD		Sensor6;
	DWORD		Sensor7;
} VTM_SCANAPI_DEVICE_SENSORSTATUS, *PVTM_SCANAPI_DEVICE_SENSORSTATUS;

typedef struct tagVTM_STATUS_SENSOR
{
	VTM_STATUS_SENSOR_TYPE	wSensor[VTM_STATUS_SENSOR_COUNT];
	BYTE					bySensorBit;
} VTM_STATUS_SENSOR, *PVTM_STATUS_SENSOR;

typedef struct tagVTM_STATUS_HARDWARE
{
	VTM_STATUS_HARDWARE_TYPE	wHardward[VTM_STATUS_HARDWARE_COUNT];
} VTM_STATUS_HARDWARE, *PVTM_STATUS_HARDWARE;

typedef struct tagVTM_STATUS_INFO
{
	VTM_STATUS_SPEED		Speed;
	VTM_STATUS_LED			Led;
	VTM_STATUS_SENSOR		Sensor;
	VTM_STATUS_SKEW			Skew;
	VTM_STATUS_HARDWARE		Hardware;
} VTM_STATUS_INFO, *PVTM_STATUS_INFO;

typedef struct tagVTM_SPEED_PARAM
{
	VTM_SPEED_TYPE			Type;
	VTM_SPEED_SPEED			Speed;
} VTM_SPEED_PARAM, *PVTM_SPEED_PARAM;

typedef struct tagVTM_DEVICE_PARAM
{
	VTM_DEVICE_PARAM_TYPE	Type;
	union
	{
		VTM_SPEED_PARAM		Speed;
		VTM_EJECT_TYPE		EjectType;
		VTM_MOTOR_ACTION	MotorAction;
	} info;
} VTM_DEVICE_PARAM, *PVTM_DEVICE_PARAM;

typedef struct tagVTM_SCAN_PARAM
{
	BOOL	bBackwardScan;
	WORD	wScanType;
	WORD	wScanMode;
	WORD	wResolution;
	DWORD	dwX0;
	DWORD	dwY0;
	DWORD	dwCX;
	DWORD	dwCY;
	WORD	wFileFormat;
	TCHAR	szFront[MAX_PATH];
	TCHAR	szRear[MAX_PATH];
	BOOL	bSkipFeeding;
	BOOL    bNarrowPaper;
	BOOL	bMultiFeed;
	BOOL	bReserved[2];
} VTM_SCAN_PARAM, *PVTM_SCAN_PARAM;

typedef struct tagVTM_IMAGE_PARAM
{
	int		iBrightness;
	int		iContrast;
	int		iGamma;
	WORD	wQuality;
	BOOL	bAutoSize;
	BOOL	bAutoDensity;
	BOOL	bRemoveBlankPage;
	BOOL	bRemoveBackground;
	BOOL	bRemovePunchHoles;
	BOOL	bAutoEnhance;
	BOOL	bCharacterEnhancement;
	BOOL	bImageEdgeFill;
	BOOL	bBarcode;
	WORD	wBarcodeCategory;
	WORD	wReserved;
	BOOL	bReserved[3];
} VTM_IMAGE_PARAM, *PVTM_IMAGE_PARAM;

typedef struct tagVTM_BARCODE_INFO
{
	VTM_BARCODE_TYPE  Type;
	TCHAR			  szText[BARCODE_MAX_LENGTH];
} VTM_BARCODE_INFO, *PVTM_BARCODE_INFO;

typedef struct tagVTM_BARCODE_INFO_BIG
{
	VTM_BARCODE_TYPE  Type;
	TCHAR			  szText[BARCODE_BIG_MAX_LENGTH];
} VTM_BARCODE_INFO_BIG, *PVTM_BARCODE_INFO_BIG;

typedef struct tagVTM_PAPER_SIZE
{
	WORD	wWidth;
} VTM_PAPER_SIZE, *PVTM_PAPER_SIZE;

typedef void (*CB_FUNC)(CB_INFO);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_Init(CB_FUNC CBFunc);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_DeInit();
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetDeviceList(SCANNERINFO *ScanInfo);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_OpenScanner(char* dev_name);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_DoCalibration();
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_SetProperty(char* szInData);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_Scan();
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetPaperStatus(PLKSS_PAPER_STATUS *Paper_Status);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_ManualEjectPaper(EJECT_DIRECTION Direction);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_CloseScanner();
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetDriverVersion(char *szVersion);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MergeImg(char *szFileList, char *szOutFileName, int iImgFmt, char *szLangList, CB_INFO *CBInfo);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetDevStatus();
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_StopScan();
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetFSensorStatus();
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetBSensorStatus();
// PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_VTM_BarcodeRead(WORD* iicount1, PVTM_BARCODE_INFO_O* pBarcodeInfo1, WORD* iicount2, PVTM_BARCODE_INFO_O* pBarcodeInfo2);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_VTM_BarcodeRead(VTM_BARCODEDATA *BarcodeData);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_VTM_DoSpeed(VTM_SPEEDPARAM SpeedParam);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetDeviceListEx(SCANNERINFOEX *ScanInfoEx);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetProperty(char* szInData);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_ResetScanner();
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetButtonStatus(PBYTE pbtn_code);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetSerialNumber(DEVICE_SERIAL_NUMBER *device_serial_number);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MergeImgEx(char *szFileList, char *szOutFileName, int iImgFmt, char *szLangList, CB_INFO *CBInfo, int bDelOriFiles);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetVersionInfo(int iInSizeofszOutVerInfoJsonString, char* szOutVerInfoJsonString, int* iOutLengthOfJsonString);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetBarcodeData(char *szFileName, int bGenFile, char **szOutBarcodeData, int *iOutLengthOfBarcodeData);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_ReadImageByFileName(char *szFileName);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_GetDeviceCapability(char *szInDevName, char **szOutCapJsonString);

/* multi-devices */
typedef void* DevHandle;

PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiInit(DevHandle* hDevHandle, CB_FUNC CBFunc);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiDeInit(DevHandle* hDevHandle);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiGetDeviceList(DevHandle* hDevHandle, SCANNERINFO *ScanInfo);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiOpenScanner(DevHandle* hDevHandle, char* dev_name);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiDoCalibration(DevHandle* hDevHandle);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiSetProperty(DevHandle* hDevHandle, char* szInData);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiScan(DevHandle* hDevHandle);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiGetPaperStatus(DevHandle* hDevHandle, PLKSS_PAPER_STATUS *Paper_Status);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiManualEjectPaper(DevHandle* hDevHandle, EJECT_DIRECTION Direction);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiCloseScanner(DevHandle* hDevHandle);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiGetDriverVersion(DevHandle* hDevHandle, char *szVersion);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiMergeImg(DevHandle* hDevHandle, char *szFileList, char *szOutFileName, int iImgFmt, char *szLangList, CB_INFO *CBInfo);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiGetDevStatus(DevHandle* hDevHandle);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiStopScan(DevHandle* hDevHandle);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiGetFSensorStatus(DevHandle* hDevHandle);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiGetBSensorStatus(DevHandle* hDevHandle);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiVTM_BarcodeRead(DevHandle* hDevHandle, VTM_BARCODEDATA *BarcodeData);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiVTM_DoSpeed(DevHandle* hDevHandle, VTM_SPEEDPARAM SpeedParam);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiGetDeviceListEx(DevHandle* hDevHandle, SCANNERINFOEX *ScanInfoEx);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiGetProperty(DevHandle* hDevHandle, char* szInData);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiResetScanner(DevHandle* hDevHandle);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiDeInitEx(DevHandle* hDevHandle);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiGetButtonStatus(DevHandle* hDevHandle, PBYTE pbtn_code);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiMergeImgEx(DevHandle* hDevHandle, char *szFileList, char *szOutFileName, int iImgFmt, char *szLangList, CB_INFO *CBInfo, int bDelOriFiles);
PLK_SCANSDK_API ENUM_PLKSS_ERRCODE PSS_MultiGetVersionInfo(DevHandle* hDevHandle, int iInSizeofszOutVerInfoJsonString, char* szOutVerInfoJsonString, int* iOutLengthOfJsonString);
extern void* ButtonHandler(void);
extern void* mButtonHandler(DevHandle* hDevHandle);

//VTM API 2.0
PLK_SCANSDK_API VTM_RET PSS_VTM2_OpenScanner(VTM_OPEN_LEVEL Level, VTM_STATUS_RET * StatusRet);
PLK_SCANSDK_API VTM_RET PSS_VTM2_CloseScanner();
PLK_SCANSDK_API VTM_RET PSS_VTM2_GetVersionInfo(VTM_VERSION_INFO * VersionInfo);
PLK_SCANSDK_API VTM_RET PSS_VTM2_GetStatusInfo(VTM_STATUS_INFO * StatusInfo, VTM_STATUS_RET * StatusRet);
PLK_SCANSDK_API VTM_RET PSS_VTM2_SetDeviceParam(VTM_DEVICE_PARAM DeviceParam, VTM_STATUS_RET * StatusRet);
PLK_SCANSDK_API VTM_RET PSS_VTM2_Scan(VTM_SCAN_PARAM ScanParam, VTM_IMAGE_PARAM ImageParam, VTM_STATUS_RET * StatusRet);
PLK_SCANSDK_API VTM_RET PSS_VTM2_GetBarcodeResult(WORD * wNumber1, PVTM_BARCODE_INFO * pBarcodeInfo1, WORD * wNumber2, PVTM_BARCODE_INFO * pBarcodeInfo2);
PLK_SCANSDK_API VTM_RET PSS_VTM2_UpgradeFirmware();
PLK_SCANSDK_API VTM_RET PSS_VTM2_GetBarcodeResultBig(WORD * wNumber1, PVTM_BARCODE_INFO_BIG * pBarcodeInfo1, WORD * wNumber2, PVTM_BARCODE_INFO_BIG * pBarcodeInfo2);
PLK_SCANSDK_API VTM_RET PSS_VTM2_CheckPaperSize(VTM_PAPER_SIZE * PaperSize, VTM_STATUS_RET * StatusRet);
PLK_SCANSDK_API VTM_RET PSS_VTM2_DoCalibration(VTM_STATUS_RET * StatusRet);
  
#ifdef __cplusplus
}
#endif

#endif
